/**
Interface für Datenelemente

- author: Klaus Reinold
- version: 1.0
*/
protocol Datenelement
{
    /** Methode zur Ausgabe von Information über das Datenelement */
    func InformationAusgeben()
    
}

/**
Beschreibung der Klasse Patient
 
- author: Klaus Reinold
- version: 1.0
*/
class Patient: Datenelement
{
    /** Name des Patienten */
    private var name: String

    /**
    Konstruktor des Patienten
    - parameters:
        - s: Name des Patienten
    */
    init(s: String)
    {
        name = s
    }

    /**
    Gibt Information über den Patienten aus.
    */
    func InformationAusgeben()
    {
        print(name)
    }

    /**
    Gibt den Namen des Patienten aus
    - returns: Name des Patienten
    */
    func NameGeben() -> String
    {
        return name
    }
}

/**
Beschreibung der Klasse Knoten

- author: Klaus Reinold
- version: 1.0
*/
class Knoten
{
    /** Verwaltung des nachfolgenden Knoten in der Warteschlange. */
    private var nachfolger: Knoten?
    /** Verwaltung der Daten */
    private var daten: Datenelement

    /**
    Konstruktor des Knoten
    - parameters:
        - dNeu: Datenelement
    */
    init(dneu: Datenelement)
    {
        nachfolger  = nil
        daten = dneu
    }

    /**
    Fügt ein Datenelement am Ende der Liste ein
    - parameters:
        - dneu: neues Datenelement
    */
    func HintenEinfügen(dneu: Datenelement)
    {
        if nachfolger != nil
        {
            nachfolger!.HintenEinfügen(dneu: dneu)
        }
        else
        {
            let kneu = Knoten(dneu: dneu)
            nachfolger = kneu
        }
    }

    /**
    Nachfolger des Knoten setzen
    - parameters:
        - nf: neuer Nachfolger
    */
    func NachfolgerSetzen(nf: Knoten)
    {
        nachfolger = nf
    }

    /**
    Gibt den Nachfolger des Knoten aus.
    - returns: Nachfolger
    */
    func NachfolgerGeben() -> Knoten?
    {
        return nachfolger
    }
    
    /**
    Gibt das Datenelement zurück
    - returns: Datenelement
    */
    func DatenelementGeben() -> Datenelement
    {
        return daten
    }
    
    /**
    Gibt Information über das Datenelement aus.
    */
    func InformationAusgeben()
    {
        daten.InformationAusgeben()
    }
}

/**
Beschreibung der Warteschlange

- author: Klaus Reinold
- version: 1.0
*/
class Warteschlange
{
    /** erster Knoten der Warteschlange */
    private var anfang: Knoten?

    /**
    Konstruktor der Warteschlange
    */
    init()
    {
        anfang = nil
    }

    /**
     Fügt ein neues Datenelement am Ende der Warteschlange ein.
    - parameters:
        - dNeu: neues Datenelement
    */
    func HintenEinfügen(dneu: Datenelement)
    {
        if anfang != nil
        {
            anfang?.HintenEinfügen(dneu: dneu)
        }
        else
        {
            anfang = Knoten(dneu: dneu)
        }
    }

    /**
    Entfernt den ersten Patient aus der Warteschlange und gibt ihn zurück.
    - returns: bisheriger erster Patient
    */
    func Entfernen() -> Datenelement?
    {
        return nil
// Die Lösung muss überarbeitet werden, weil die Referenz auf das Ende
// nicht mehr vorhanden ist.
//        let k = anfang
//        if anfang != nil
//        {
//            if anfang === ende
//            {
//                anfang = nil
//                ende = nil
//            }
//            else
//            {
//                anfang=anfang!.NachfolgerGeben()
//            }
//            return k!.DatenelementGeben()
//        }
//        else
//        {
//            return nil
//        }
    }

    /**
    Gibt Information über die Warteschlange aus.
    */
    func InformationAusgeben()
    {
        if anfang != nil
        {
            anfang!.InformationAusgeben()
        }
    }
}
